/*!
 *
 */

//!
#ifndef RFCC_H
#define RFCC_H

#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

#include "settings.h"

#include "spi.h"

//! Configuration Registers
//@{
#define RFCC_IOCFG2       0x00        // GDO2 output pin configuration
#define RFCC_IOCFG1       0x01        // GDO1 output pin configuration
#define RFCC_IOCFG0       0x02        // GDO0 output pin configuration
#define RFCC_FIFOTHR      0x03        // RX FIFO and TX FIFO thresholds
#define RFCC_SYNC1        0x04        // Sync word, high byte
#define RFCC_SYNC0        0x05        // Sync word, low byte
#define RFCC_PKTLEN       0x06        // Packet length
#define RFCC_PKTCTRL1     0x07        // Packet automation control
#define RFCC_PKTCTRL0     0x08        // Packet automation control
#define RFCC_ADDR         0x09        // Device address
#define RFCC_CHANNR       0x0A        // Channel number
#define RFCC_FSCTRL1      0x0B        // Frequency synthesizer control
#define RFCC_FSCTRL0      0x0C        // Frequency synthesizer control
#define RFCC_FREQ2        0x0D        // Frequency control word, high byte
#define RFCC_FREQ1        0x0E        // Frequency control word, middle byte
#define RFCC_FREQ0        0x0F        // Frequency control word, low byte
#define RFCC_MDMCFG4      0x10        // Modem configuration
#define RFCC_MDMCFG3      0x11        // Modem configuration
#define RFCC_MDMCFG2      0x12        // Modem configuration
#define RFCC_MDMCFG1      0x13        // Modem configuration
#define RFCC_MDMCFG0      0x14        // Modem configuration
#define RFCC_DEVIATN      0x15        // Modem deviation setting
#define RFCC_MCSM2        0x16        // Main Radio Cntrl State Machine config
#define RFCC_MCSM1        0x17        // Main Radio Cntrl State Machine config
#define RFCC_MCSM0        0x18        // Main Radio Cntrl State Machine config
#define RFCC_FOCCFG       0x19        // Frequency Offset Compensation config
#define RFCC_BSCFG        0x1A        // Bit Synchronization configuration
#define RFCC_AGCCTRL2     0x1B        // AGC control
#define RFCC_AGCCTRL1     0x1C        // AGC control
#define RFCC_AGCCTRL0     0x1D        // AGC control
#define RFCC_WOREVT1      0x1E        // High byte Event 0 timeout
#define RFCC_WOREVT0      0x1F        // Low byte Event 0 timeout
#define RFCC_WORCTRL      0x20        // Wake On Radio control
#define RFCC_FREND1       0x21        // Front end RX configuration
#define RFCC_FREND0       0x22        // Front end TX configuration
#define RFCC_FSCAL3       0x23        // Frequency synthesizer calibration
#define RFCC_FSCAL2       0x24        // Frequency synthesizer calibration
#define RFCC_FSCAL1       0x25        // Frequency synthesizer calibration
#define RFCC_FSCAL0       0x26        // Frequency synthesizer calibration
#define RFCC_RCCTRL1      0x27        // RC oscillator configuration
#define RFCC_RCCTRL0      0x28        // RC oscillator configuration
#define RFCC_FSTEST       0x29        // Frequency synthesizer cal control
#define RFCC_PTEST        0x2A        // Production test
#define RFCC_AGCTEST      0x2B        // AGC test
#define RFCC_TEST2        0x2C        // Various test settings
#define RFCC_TEST1        0x2D        // Various test settings
#define RFCC_TEST0        0x2E        // Various test settings
//@}

//! Strobe commands
//@{
#define RFCC_SRES         0x30        // Reset chip.
#define RFCC_SFSTXON      0x31        // Enable/calibrate freq synthesizer
#define RFCC_SXOFF        0x32        // Turn off crystal oscillator.
#define RFCC_SCAL         0x33        // Calibrate freq synthesizer & disable
#define RFCC_SRX          0x34        // Enable RX.
#define RFCC_STX          0x35        // Enable TX.
#define RFCC_SIDLE        0x36        // Exit RX / TX
#define RFCC_SAFC         0x37        // AFC adjustment of freq synthesizer
#define RFCC_SWOR         0x38        // Start automatic RX polling sequence
#define RFCC_SPWD         0x39        // Enter pwr down mode when CSn goes hi
#define RFCC_SFRX         0x3A        // Flush the RX FIFO buffer.
#define RFCC_SFTX         0x3B        // Flush the TX FIFO buffer.
#define RFCC_SWORRST      0x3C        // Reset real time clock.
#define RFCC_SNOP         0x3D        // No operation.
//@}

//! Status registers
//@{
#define RFCC_PARTNUM      0x30        // Part number
#define RFCC_VERSION      0x31        // Current version number
#define RFCC_FREQEST      0x32        // Frequency offset estimate
#define RFCC_LQI          0x33        // Demodulator estimate for link quality
#define RFCC_RSSI         0x34        // Received signal strength indication
#define RFCC_MARCSTATE    0x35        // Control state machine state
#define RFCC_WORTIME1     0x36        // High byte of WOR timer
#define RFCC_WORTIME0     0x37        // Low byte of WOR timer
#define RFCC_PKTSTATUS    0x38        // Current GDOx status and packet status
#define RFCC_VCO_VC_DAC   0x39        // Current setting from PLL cal module
#define RFCC_TXBYTES      0x3A        // Underflow and # of bytes in TXFIFO
#define RFCC_RXBYTES      0x3B        // Overflow and # of bytes in RXFIFO
#define RFCC_NUM_RXBYTES  0x7F        // Mask "# of bytes" field in _RXBYTES
//@}

//! Other memory locations
//@{
#define RFCC_PATABLE      0x3E
#define RFCC_TXFIFO       0x3F
#define RFCC_RXFIFO       0x3F
//@}

//! Masks for appended status bytes
//@{
#define RFCC_LQI_RX       0x01        // Position of LQI byte
#define RFCC_CRC_OK       0x80        // Mask "CRC_OK" bit within LQI byte
//@}

//! Definitions to support burst/single access:
//@{
#define RFCC_WRITE_BURST  0x40
#define RFCC_READ_SINGLE  0x80
#define RFCC_READ_BURST   0xC0
//@}

// void RFSendPacket(char *txBuffer, char size);
// char RFReceivePacket(char *rxBuffer, char *length);
// void RF_TX_proc(void);
// void RF_RX_proc(void);

namespace rfcc
{
	typedef struct settings_s
	{
		char FSCTRL0;
		char FREQ2;
		char FREQ1;
		char FREQ0;
		char MDMCFG4;
		char MDMCFG3;
		char MDMCFG2;
		char MDMCFG1;
		char MDMCFG0;
		char CHANNR;
		char DEVIATN;
		char FREND1;
		char FREND0;
		char MCSM2;
		char MCSM1;
		char MCSM0;
		char FOCCFG;
		char BSCFG;
		char AGCCTRL2;
		char AGCCTRL1;
		char AGCCTRL0;
		char FSCAL3;
		char FSCAL2;
		char FSCAL1;
		char FSCAL0;
		char FSTEST;
		char TEST2;
		char TEST1;
		char TEST0;
		char IOCFG2;
		char IOCFG0;
		char PKTCTRL1;
		char PKTCTRL0;
		char ADDR;
		char PKTLEN;
		char FIFOTHR;
	} settings_t;

	// PATABLE (0 dBm output power)
	char paTable[] = {0xff};
	char paTableLen = 1;
	char LQI, RSSI;

	/*!
	 * Configures the assigned interface to function as a SPI port and
	 * initializes it.
	 */
	void init(settings_t *settings);

	void powerupReset();

	/*!
	 * This function transmits a packet with length up to 63 bytes.  To use this
	 * function, GD00 must be configured to be asserted when sync word is sent and
	 * de-asserted at the end of the packet, which is accomplished by setting the
	 * IOCFG0 register to 0x06, per the CCxxxx datasheet.  GDO0 goes high at
	 * packet start and returns low when complete.  The function polls GDO0 to
	 * ensure packet completion before returning.
	 * @param *txBuffer Pointer to a buffer containing the data to be transmitted
	 * @param size The size of the txBuffer
	 */
	void sendPacket(char *txBuffer, char size);

	/*!
	 * Receives a packet of variable length (first byte in the packet must be the
	 * length byte).  The packet length should not exceed the RXFIFO size.  To use
	 * this function, APPEND_STATUS in the PKTCTRL1 register must be enabled.  It
	 * is assumed that the function is called after it is known that a packet has
	 * been received; for example, in response to GDO0 going low when it is
	 * configured to output packet reception status.
	 *
	 * The RXBYTES register is first read to ensure there are bytes in the FIFO.
	 * This is done because the GDO signal will go high even if the FIFO is flushed
	 * due to address filtering, CRC filtering, or packet length filtering.
	 * @param *rxBuffer Pointer to the buffer where the incoming data should be stored
	 * @param *length Pointer to a variable containing the size of the buffer where the 
	 *                incoming data should be stored. After this function returns, that 
	 *                variable holds the packet length.
	 * @returns \b 0x80:  CRC OK \n
	 *          \b 0x00:  CRC NOT OK (or no pkt was put in the RXFIFO due to filtering)
	 */
	char receivePacket(char *rxBuffer, char *length);

	namespace rfspi
	{
		/*!
		 * Configures the assigned interface to function as a SPI port and
		 * initializes it.
		 */
		void init();
		
		/*!
		 * Writes "value" to a single configuration register at address "addr".
		 * @param addr register address
		 * @param value new register value
		 */
		void writeReg(char addr, char value);

		/*!
		 * Writes values to multiple configuration registers, the first register being
		 * at address "addr".  First data byte is at "buffer", and both addr and
		 * buffer are incremented sequentially (within the CCxxxx and AVR,
		 * respectively) until "count" writes have been performed.
		 * @param addr first register address
		 * @param *buffer ptr to array of bytes
		 * @param count array length
		 */
		void writeBurstReg(char addr, char *buffer, char count);

		/*!
		 * Reads a single configuration register at address "addr" and returns the
		 * value read.
		 * @param addr register address
		 * @return register value
		 */
		char readReg(char addr);

		/*!
		 * Reads multiple configuration registers, the first register being at address
		 * "addr".  Values read are deposited sequentially starting at address
		 * "buffer", until "count" registers have been read.
		 * @param addr first register address
		 * @param *buffer ptr to array of bytes
		 * @param count array length
		 */
		void readBurstReg(char addr, char *buffer, char count);

		/*!
		 * Special read function for reading status registers.  Reads status register
		 * at register "addr" and returns the value read.
		 * @param addr status register address
		 * @return status
		 */
		char readStatus(char addr);

		/*!
		 * Special write function for writing to command strobe registers.  Writes
		 * to the strobe at address "addr".
		 * @param strobe_ "addr"
		 */
		void strobe(char strobe_);
	};
};

#endif // RF_CC_H
